// AP(r) Computer Science Marine Biology Simulation:
// The MBSGUIFrame class is copyright(c) 2002 College Entrance
// Examination Board (www.collegeboard.com).
//
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.HyperlinkEvent;
import java.util.ArrayList;
import java.util.Iterator;
import java.lang.reflect.Method;

/**
 *  AP&reg; Computer Science Marine Biology Simulation:<br>
 *  A <code>MBSGUIFrame</code> is a window containing the graphical display of
 *  the simulation and menus/buttons that allow the user to
 *  edit the environment and control the simulation.
 *
 *  <p>
 *  The <code>MBSGUIFrame</code> class is
 *  copyright&copy; 2002 College Entrance Examination Board
 *  (www.collegeboard.com).
 *
 *  @author Julie Zelenski
 *  @author Chris Nevison
 *  @version 1 August 2002
 **/
public class MBSGUIFrame extends JFrame
{
    private static final String VERSION_ID = "1.0)";
    private static final String VERSION_DATE = "1 August 2002";
    
    private EnvironmentController envControl;
    private SimulationController simControl;
    private MBSDisplay display;
    private ArrayList menuItemsThatNeedAnEnvironment, menuItemsDisabledDuringRun;


    /** Creates a new MBSGUIFrame and all its controls.
     **/
    public MBSGUIFrame()
    {
        System.setProperty("sun.awt.exception.handler", GUIExceptionHandler.class.getName());
        
        setTitle("Marine Biology Simulation");
        setLocation(25, 15);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
    
        JPanel content = new JPanel();
        setContentPane(content);
        content.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
        content.setLayout(new BorderLayout());
    
        display = new MBSDisplay();
        JScrollPane scrollPane = new JScrollPane();
        scrollPane.setViewport(new PseudoInfiniteViewport(scrollPane));
        scrollPane.setViewportView(display);
        content.add(scrollPane, BorderLayout.CENTER);
    
        simControl = new SimulationController(this, display);
        content.add(simControl.controlPanel(), BorderLayout.SOUTH);
        
        envControl = new EnvironmentController(this, display, simControl);
        content.add(envControl.controlPanel(), BorderLayout.EAST);

        makeMenus();
        pack();
    }

    /** Creates the drop-down menus on the frame. 
     **/
    private void makeMenus()
    {
        int menuMask = getToolkit().getMenuShortcutKeyMask();
        JMenuBar mbar = new JMenuBar();
        JMenu menu;
        JMenuItem mItem;
        
        menuItemsThatNeedAnEnvironment = new ArrayList();
        menuItemsDisabledDuringRun = new ArrayList();
        
        mbar.add(menu = new JMenu("File"));
        menu.add(mItem = new JMenuItem("Open environment file..."));
        mItem.addActionListener( new ActionListener() { 
            public void actionPerformed(ActionEvent e) { envControl.openFile(); }});
        mItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, menuMask));
        
        menu.add(mItem = new JMenuItem("Create new environment..."));
        mItem.addActionListener( new ActionListener() { 
            public void actionPerformed(ActionEvent e) { envControl.createNewEnv(); }});
        mItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, menuMask));
        
        menu.add(mItem = new JMenuItem("Edit environment..."));
        mItem.addActionListener( new ActionListener() { 
            public void actionPerformed(ActionEvent e) { envControl.editEnv(); }});
        mItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, menuMask));
        menuItemsThatNeedAnEnvironment.add(mItem);

        menu.add(mItem  = new JMenuItem("Save environment as..."));
        mItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, menuMask));
        mItem.addActionListener( new ActionListener() { 
            public void actionPerformed(ActionEvent e) { envControl.saveFile(); }});
        menuItemsThatNeedAnEnvironment.add(mItem);

        menu.add(mItem = new JMenuItem("Quit"));
        mItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, menuMask));
        mItem.addActionListener( new ActionListener() { 
            public void actionPerformed(ActionEvent e) { System.exit(0); }});

        mbar.add(menu = new JMenu("Seed"));
        final JMenu seedMenu = menu;
        ButtonGroup bGroup = new ButtonGroup();
        menu.add(mItem = new JRadioButtonMenuItem("Don't reset seed", true));
        bGroup.add(mItem);
        mItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                envControl.setSeedOption(envControl.DONT_RESET_SEED); }});
        menu.add(mItem = new JRadioButtonMenuItem("Use fixed seed..."));
        bGroup.add(mItem);
        mItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                int result = envControl.setSeedOption(envControl.FIXED_SEED);
                seedMenu.getItem(result).setSelected(true); }});
        menu.add(mItem = new JRadioButtonMenuItem("Prompt for seed"));
        bGroup.add(mItem);
        mItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                envControl.setSeedOption(envControl.PROMPT_SEED); }});

        mbar.add(menu = new JMenu("Run"));
        final JMenu runMenu = menu;
        bGroup = new ButtonGroup();
        menu.add(mItem = new JRadioButtonMenuItem("Run Indefinitely", true));
        bGroup.add(mItem);
        mItem.addActionListener( new ActionListener() { 
            public void actionPerformed(ActionEvent e) { 
                simControl.setRunOption(simControl.INDEFINITE); }});
        menuItemsDisabledDuringRun.add(mItem);
        
        menu.add(mItem = new JRadioButtonMenuItem("Use fixed number of steps..."));
        bGroup.add(mItem);
        mItem.addActionListener( new ActionListener() { 
            public void actionPerformed(ActionEvent e) { 
                int result = simControl.setRunOption(simControl.FIXED_STEPS);
                runMenu.getItem(result).setSelected(true); }});
        menuItemsDisabledDuringRun.add(mItem);
        
        menu.add(mItem = new JRadioButtonMenuItem("Prompt for number of steps"));
        bGroup.add(mItem);
        mItem.addActionListener( new ActionListener() { 
            public void actionPerformed(ActionEvent e) { 
                simControl.setRunOption(simControl.PROMPT_STEPS); }});
        menuItemsDisabledDuringRun.add(mItem);

        mbar.add(menu = new JMenu("View"));
        menu.add(mItem = new JMenuItem("Zoom in"));
        mItem.addActionListener( new ActionListener() { 
            public void actionPerformed(ActionEvent e) { display.zoomIn(); }});
        mItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_UP, menuMask));
        menuItemsThatNeedAnEnvironment.add(mItem);

        menu.add(mItem = new JMenuItem("Zoom out"));
        mItem.addActionListener( new ActionListener() { 
            public void actionPerformed(ActionEvent e) { display.zoomOut(); }});
        mItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, menuMask));
        menuItemsThatNeedAnEnvironment.add(mItem);

        menu.add(mItem = new JMenuItem("Bring (0, 0) to upper left"));
        mItem.addActionListener( new ActionListener() { 
            public void actionPerformed(ActionEvent e) { display.recenterOnOrigin(); }});
        mItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, menuMask));
        menuItemsThatNeedAnEnvironment.add(mItem);
        
        // This inspects the Fish class to see if there is a static method
        // enableBreedDie(boolean) and if so, attaches it to a menu item.
        try 
        {
            final Method m = Fish.class.getMethod("enableBreedDie", new Class[]{boolean.class});
            m.invoke(Fish.class, new Object[] { new Boolean(true) });
            mbar.add(menu = new JMenu("Special"));
            final JMenu specialMenu = menu;
            bGroup = new ButtonGroup();
            menu.add(mItem = new JRadioButtonMenuItem("Enable breed and die", true));
            bGroup.add(mItem);
            mItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) { 
                try 
                { 
                    m.invoke(Fish.class, new Object[] { new Boolean(true) });
                } 
                catch (Exception e) {} 
                }});
            menu.add(mItem = new JRadioButtonMenuItem("Disable breed and die"));
            bGroup.add(mItem);
            mItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) { 
                try 
                { 
                    m.invoke(Fish.class, new Object[] { new Boolean(false) });
                } 
                catch (Exception e) {} 
                }});
        }
        catch (Exception e) {}   // quietly ignore any problem setting up the special menu

        mbar.add(menu = new JMenu("Help"));
        menu.add(mItem = new JMenuItem("About MBSGUI..."));
        mItem.addActionListener( new ActionListener() { 
            public void actionPerformed(ActionEvent e) { showAboutPanel(); }});
        menu.add(mItem = new JMenuItem("Help..."));
        mItem.addActionListener( new ActionListener() { 
            public void actionPerformed(ActionEvent e) { showHelp(); }});
        mItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_HELP, menuMask));

        setEnvMenuItemsEnabled(false);
        setRunMenuItemsEnabled(true);
        setJMenuBar(mbar);
    }
    
    /** Sets the enabled status of those menu items that need an
     *  environment to be valid. 
     **/
    public void setEnvMenuItemsEnabled(boolean enable)
    {
        Iterator iter = menuItemsThatNeedAnEnvironment.iterator();
        while (iter.hasNext())
            ((JMenuItem)iter.next()).setEnabled(enable);
    }
    
    /** Sets the enabled status of those menu items that are
     *  disabled when running. 
     **/
    public void setRunMenuItemsEnabled(boolean enable)
    {
        Iterator iter = menuItemsDisabledDuringRun.iterator();
        while (iter.hasNext())
            ((JMenuItem)iter.next()).setEnabled(enable);
    }
   /** Brings up a simple dialog with some general information.
     **/
    private void showAboutPanel()
    {
        String html = "<html><h2>Marine Biology Simulation Program<br>" +
                    "with Graphical User Interface (MBSGUI)</h2>" +
                    "A tool for running and viewing the AP&reg; Computer Science<p>" +
                    "Marine Biology Simulation case study program.<p><p>" +
                    "<font size=-1>Version: " + VERSION_DATE + "</font><p>" +
                    "<font size=-1>Copyright&copy; 2002 College Entrance Examination Board " +
                    "(www.collegeboard.com).</font></font></html>";
        JOptionPane.showMessageDialog(this, new JLabel(html), "About MBSGUI", 
                JOptionPane.INFORMATION_MESSAGE, 
                new DrawableToolbar.DrawableIcon(Drawable.class, Color.magenta, 48, 48));
    }
    
    /** Brings up a window with a scrolling text pane that display
     *  the help information for the simulation.
     **/
    private void showHelp()
    {
        JDialog dialog = new JDialog(this, "MBSGUI Help");
        final JEditorPane helpText = new JEditorPane();
        try 
        {
            helpText.setPage(getClass().getResource("MBSHelp.html"));
        } 
        catch (Exception e)
        {
            helpText.setText("Couldn't load help file.");
        }
        helpText.setEditable(false);
        helpText.addHyperlinkListener(new HyperlinkListener() {
            public void hyperlinkUpdate(HyperlinkEvent ev) {
                if (ev.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
                    try 
                    {
                        helpText.setPage(ev.getURL());
                    }
                    catch (Exception ex) {}
            }});
        JScrollPane sp = new JScrollPane(helpText);
        sp.setPreferredSize(new Dimension(650, 500));
        dialog.getContentPane().add(sp);
        dialog.setLocation(getX() + getWidth() - 200, getY() + 50);
        dialog.pack();
        dialog.setVisible(true);
    }
    
    /** Nested class that is registered as the handler for exceptions
     *  on the Swing event thread. The handler will put up an alert panel,
     *  dump the stack trace to the console, and then exit entire program 
     *  rather than persist in an inconsistent state, which would be
     *  the default behavior.
     */
    public static class GUIExceptionHandler
    {
        public void handle(Throwable e)
        {
            e.printStackTrace();
            JOptionPane.showMessageDialog(null, 
                "An error occurred. The simulation must exit." + "\nReason: " + e, 
                "Error", JOptionPane.ERROR_MESSAGE);
            System.exit(0);
        }
    }
}


